@@ -9,26 +9,29 @@ Features
 --------
 
 - Read in Phantom HDF snapshot files.
-- Read in global quantity evolution files.
+- Read in global quantity and sink evolution files.
 - Encapsulate entire simulation data as Simulation object.
 - Access particle and sink arrays.
 - Access simulation parameters and units.
 - Compute extra quantities on particles.
 - Visualize data using kernel interpolation.
+- Generate radial profiles.
 
 Classes
 -------
 
+- Profile
+    Represents a radial profile through the snapshot in either
+    cylindrical or spherical coordinates.
+
 - Snap
     Represents a smoothed particle hydrodynamics snapshot file,
     containing particles, sinks, and file header information.
 
-- Evolution
-    Represents globally averaged quantities as time series data.
-
 - Simulation
     Represents an entire smoothed particle hydrodynamics simulation.
-    It contains instances of Snap and Evolution objects.
+    It contains a list of Snap objects, and time series data as pandas
+    dataframes.
 
 - Visualization
     Represents a visualization of a Snap object.
@@ -69,25 +72,18 @@ units: Any = pint.UnitRegistry(system='cgs')
 Quantity: Any = units.Quantity
 
 from . import analysis, simulation, snap, utils, visualize
-from .analysis import Profile
-from .simulation import Evolution, Simulation, load_ev, load_sim
+from .analysis import Profile, load_profile
+from .simulation import Simulation, load_ev, load_sim
 from .snap import Snap, load_snap
 from .visualize import Visualization
 
 
 __all__ = (
-    ['Evolution', 'Profile', 'Simulation', 'Snap', 'Visualization']  # Classes
+    ['Profile', 'Simulation', 'Snap', 'Visualization']  # Classes
     + ['analysis', 'simulation', 'snap', 'utils', 'visualize']  # Packages
-    + ['load_snap', 'load_ev', 'load_sim']  # User functions
+    + ['load_ev', 'load_profile', 'load_snap', 'load_sim']  # User functions
     + ['units', 'Quantity']
 )
 
 # Canonical version number
-__version__ = '0.4.0'
-
-# Add units
-units.define('solar_mass = 1.9891e33 g')
-units.define('solar_radius = 6.959500e10 cm')
-units.define('earth_mass = 5.979e27 g')
-units.define('earth_radius = 6.371315e8 cm')
-units.define('jupiter_mass = 1.89813e30 g')
+__version__ = '0.4.1'
