@@ -9,9 +9,31 @@ import numpy as np
 from numpy import ndarray
 
 from .. import Quantity
+from .. import units as plonk_units
 from ..snap import SnapLike
 from ..utils.math import cross, norm
 
+_units = {
+    'momentum': 'momentum',
+    'angular_momentum': 'angular_momentum',
+    'specific_angular_momentum': 'specific_angular_momentum',
+    'kinetic_energy': 'energy',
+    'specific_kinetic_energy': 'specific_energy',
+    'semi_major_axis': 'length',
+    'eccentricity': 'dimensionless',
+    'inclination': 'radian',
+    'gas_mass': 'mass',
+    'dust_mass': 'mass',
+    'gas_density': 'density',
+    'dust_density': 'density',
+    'radial_distance': 'length',
+    'azimuthal_angle': 'radian',
+    'polar_angle': 'radian',
+    'radial_velocity': 'velocity',
+    'angular_velocity': 'velocity',
+    'temperature': 'temperature',
+}
+
 
 def momentum(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     """Calculate the momentum.
@@ -29,7 +51,7 @@ def momentum(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The linear momentum on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         mass: ndarray = snap['mass'][h > 0]
         vel: ndarray = snap['velocity'][h > 0]
     else:
@@ -62,7 +84,7 @@ def angular_momentum(
         The angular momentum on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         mass: ndarray = snap['mass'][h > 0]
         pos: ndarray = snap['position'][h > 0]
         vel: ndarray = snap['velocity'][h > 0]
@@ -100,7 +122,7 @@ def specific_angular_momentum(
         The specific angular momentum on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         pos: ndarray = snap['position'][h > 0]
         vel: ndarray = snap['velocity'][h > 0]
     else:
@@ -129,7 +151,7 @@ def kinetic_energy(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The kinetic energy on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         mass: ndarray = snap['mass'][h > 0]
         vel: ndarray = snap['velocity'][h > 0]
     else:
@@ -155,7 +177,7 @@ def specific_kinetic_energy(snap: SnapLike, ignore_accreted: bool = False) -> nd
         The specific kinetic energy on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         vel: ndarray = snap['velocity'][h > 0]
     else:
         vel = snap['velocity']
@@ -163,6 +185,99 @@ def specific_kinetic_energy(snap: SnapLike, ignore_accreted: bool = False) -> nd
     return 1 / 2 * norm(vel, axis=1) ** 2
 
 
+def keplerian_frequency(
+    snap: SnapLike,
+    gravitational_parameter: Any,
+    origin: Union[ndarray, Tuple[float, float, float]] = (0.0, 0.0, 0.0),
+    ignore_accreted: bool = False,
+) -> ndarray:
+    """Calculate the Keplerian orbital frequency.
+
+    The Keplerian orbital frequency of particles around a mass specified
+    by gravitational parameter with an optional to specify the position
+    of the mass.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    gravitational_parameter
+        The gravitational parameter (mu = G M) as a Pint quantity.
+    origin : optional
+        The origin around which to compute the angular momentum as a
+        ndarray or tuple (x, y, z). Default is (0, 0, 0).
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    ndarray
+        The Keplerian frequency on the particles.
+    """
+    if ignore_accreted:
+        h: ndarray = snap['smoothing_length']
+        pos: ndarray = snap['position'][h > 0]
+    else:
+        pos = snap['position']
+
+    origin = np.array(origin)
+    pos = pos - origin
+
+    mu = gravitational_parameter
+    if not isinstance(pos, Quantity):
+        mu = (mu * snap.units['time'] ** 2 / snap.units['length'] ** 3).magnitude
+
+    radius = norm(pos, axis=1)
+    return np.sqrt(mu / radius ** 3)
+
+
+def stokes_number(
+    snap: SnapLike,
+    gravitational_parameter: Any,
+    origin: Union[ndarray, Tuple[float, float, float]] = (0.0, 0.0, 0.0),
+    ignore_accreted: bool = False,
+) -> ndarray:
+    """Calculate the Stokes number.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    gravitational_parameter
+        The gravitational parameter (mu = G M) as a Pint quantity.
+    origin : optional
+        The origin around which to compute the angular momentum as a
+        ndarray or tuple (x, y, z). Default is (0, 0, 0).
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    ndarray
+        The Stokes number on the particles.
+    """
+    if ignore_accreted:
+        h: ndarray = snap['smoothing_length']
+        pos: ndarray = snap['position'][h > 0]
+        t_s: ndarray = snap['stopping_time'][h > 0]
+    else:
+        pos = snap['position']
+        t_s = snap['stopping_time']
+
+    origin = np.array(origin)
+    pos = pos - origin
+
+    mu = gravitational_parameter
+    if not isinstance(pos, Quantity):
+        mu = (mu * snap.units['time'] ** 2 / snap.units['length'] ** 3).magnitude
+
+    radius = norm(pos, axis=1)
+    Omega_k = np.sqrt(mu / radius ** 3)
+
+    Stokes = t_s * Omega_k[:, np.newaxis]
+    return Stokes
+
+
 def semi_major_axis(
     snap: SnapLike,
     gravitational_parameter: Any,
@@ -180,8 +295,7 @@ def semi_major_axis(
     snap
         The Snap object.
     gravitational_parameter
-        The gravitational parameter (mu = G M). Can be a float or a Pint
-        quantity.
+        The gravitational parameter (mu = G M) as a Pint quantity.
     origin : optional
         The origin around which to compute the angular momentum as a
         ndarray or tuple (x, y, z). Default is (0, 0, 0).
@@ -194,7 +308,7 @@ def semi_major_axis(
         The semi-major axis on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         pos: ndarray = snap['position'][h > 0]
         vel: ndarray = snap['velocity'][h > 0]
     else:
@@ -204,10 +318,9 @@ def semi_major_axis(
     origin = np.array(origin)
     pos = pos - origin
 
-    if isinstance(pos, Quantity):
-        mu = gravitational_parameter.to_reduced_units()
-    else:
-        mu = gravitational_parameter
+    mu = gravitational_parameter
+    if not isinstance(pos, Quantity):
+        mu = (mu * snap.units['time'] ** 2 / snap.units['length'] ** 3).magnitude
 
     radius = norm(pos, axis=1)
 
@@ -220,9 +333,13 @@ def semi_major_axis(
 
     term = specific_energy * (specific_angular_momentum_magnitude / mu) ** 2
 
-    eccentricity = np.sqrt(1 + 2 * term.magnitude)
+    eccentricity = np.sqrt(1 + 2 * term)
 
-    return specific_angular_momentum_magnitude ** 2 / (mu * (1 - eccentricity ** 2))
+    semi_major_axis = specific_angular_momentum_magnitude ** 2 / (
+        mu * (1 - eccentricity ** 2)
+    )
+
+    return semi_major_axis
 
 
 def eccentricity(
@@ -242,8 +359,7 @@ def eccentricity(
     snap
         The Snap object.
     gravitational_parameter
-        The gravitational parameter (mu = G M). Can be a float or a Pint
-        quantity.
+        The gravitational parameter (mu = G M) as a Pint quantity.
     origin : optional
         The origin around which to compute the angular momentum as a
         ndarray or tuple (x, y, z). Default is (0, 0, 0).
@@ -256,7 +372,7 @@ def eccentricity(
         The eccentricity on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         pos: ndarray = snap['position'][h > 0]
         vel: ndarray = snap['velocity'][h > 0]
     else:
@@ -266,10 +382,9 @@ def eccentricity(
     origin = np.array(origin)
     pos = pos - origin
 
-    if isinstance(pos, Quantity):
-        mu = gravitational_parameter.to_reduced_units()
-    else:
-        mu = gravitational_parameter
+    mu = gravitational_parameter
+    if not isinstance(pos, Quantity):
+        mu = (mu * snap.units['time'] ** 2 / snap.units['length'] ** 3).magnitude
 
     radius = norm(pos, axis=1)
 
@@ -283,8 +398,6 @@ def eccentricity(
     term = specific_energy * (specific_angular_momentum_magnitude / mu) ** 2
     eccentricity = np.sqrt(1 + 2 * term)
 
-    if isinstance(pos, Quantity):
-        return eccentricity.magnitude
     return eccentricity
 
 
@@ -308,7 +421,7 @@ def inclination(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The inclination on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         mass: ndarray = snap['mass'][h > 0]
         pos: ndarray = snap['position'][h > 0]
         vel: ndarray = snap['velocity'][h > 0]
@@ -329,8 +442,45 @@ def inclination(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     return inclination
 
 
+def gas_fraction(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
+    """Calculate the gas fraction.
+
+    For dust/gas mixtures this is from the dust fraction.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    ndarray
+        The gas fraction on the particles.
+    """
+    if snap.properties['dust_method'] == 'dust/gas mixture':
+        dust_fraction: ndarray = snap['dust_fraction']
+        gas_fraction = 1 - dust_fraction.sum(axis=1)
+
+    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
+        particle_type = snap['type']
+        gas_fraction = np.ones(len(snap))
+        gas_fraction[particle_type != snap.particle_type['gas']] = 0
+
+    else:
+        gas_fraction = np.ones(len(snap))
+
+    if ignore_accreted:
+        h: ndarray = snap['smoothing_length']
+        return gas_fraction[h > 0]
+    return gas_fraction
+
+
 def gas_mass(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
-    """Calculate the gas mass from the dust fraction.
+    """Calculate the gas mass.
+
+    For dust/gas mixtures this is from the dust fraction.
 
     Parameters
     ----------
@@ -344,46 +494,26 @@ def gas_mass(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     ndarray
         The gas mass on the particles.
     """
+    if snap.properties['dust_method'] == 'dust/gas mixture':
+        dust_fraction: ndarray = snap['dust_fraction']
+        gas_fraction = 1 - dust_fraction.sum(axis=1)
+
+    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
+        particle_type = snap['type']
+        gas_fraction = np.zeros(len(snap))
+        gas_fraction[particle_type == snap.particle_type['gas']] = 1
+
+    mass: ndarray = snap['mass']
     if ignore_accreted:
-        h: ndarray = snap['smooth']
-        dustfrac: ndarray = snap['dustfrac'][h > 0]
-        mass: ndarray = snap['mass'][h > 0]
-    else:
-        dustfrac = snap['dustfrac']
-        mass = snap['mass']
-
-    gasfrac = 1 - dustfrac.sum(axis=1)
-    return gasfrac * mass
-
-
-def dust_mass(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
-    """Calculate the dust mass per species from the dust fraction.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    ndarray
-        The dust mass per species on the particles.
-    """
-    if ignore_accreted:
-        h: ndarray = snap['smooth']
-        dustfrac: ndarray = snap['dustfrac'][h > 0]
-        mass: ndarray = snap['mass'][h > 0]
-    else:
-        dustfrac = snap['dustfrac']
-        mass = snap['mass']
-
-    return dustfrac * mass[:, np.newaxis]
+        h: ndarray = snap['smoothing_length']
+        return (mass * gas_fraction)[h > 0]
+    return mass * gas_fraction
 
 
 def gas_density(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
-    """Calculate the gas density from the dust fraction.
+    """Calculate the gas density.
+
+    For dust/gas mixtures this is from the dust fraction.
 
     Parameters
     ----------
@@ -397,20 +527,100 @@ def gas_density(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     ndarray
         The gas density on the particles.
     """
-    if ignore_accreted:
-        h: ndarray = snap['smooth']
-        dustfrac: ndarray = snap['dustfrac'][h > 0]
-        density: ndarray = snap['density'][h > 0]
-    else:
-        dustfrac = snap['dustfrac']
-        density = snap['density']
+    if snap.properties['dust_method'] == 'dust/gas mixture':
+        dust_fraction: ndarray = snap['dust_fraction']
+        gas_fraction = 1 - dust_fraction.sum(axis=1)
 
-    gasfrac = 1 - dustfrac.sum(axis=1)
-    return gasfrac * density
+    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
+        particle_type = snap['type']
+        gas_fraction = np.zeros(len(snap))
+        gas_fraction[particle_type == snap.particle_type['gas']] = 1
+
+    density: ndarray = snap['density']
+    if ignore_accreted:
+        h: ndarray = snap['smoothing_length']
+        return (density * gas_fraction)[h > 0]
+    return density * gas_fraction
+
+
+def dust_fraction(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
+    """Calculate the dust fraction.
+
+    For dust/gas mixtures this exists already.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    ndarray
+        The gas fraction on the particles.
+    """
+    if snap.properties['dust_method'] == 'dust/gas mixture':
+        dust_fraction: ndarray = snap['dust_fraction']
+
+    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
+        n_dust = len(snap.properties.get('grain_size', []))
+        dust_type = snap['dust_type']
+        dust_fraction = np.zeros((len(snap), n_dust))
+        for idx in range(1, n_dust + 1):
+            dust_fraction[dust_type == idx, idx - 1] = 1
+
+    else:
+        raise ValueError('No dust available')
+
+    if ignore_accreted:
+        h: ndarray = snap['smoothing_length']
+        return dust_fraction[h > 0]
+    return dust_fraction
+
+
+def dust_mass(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
+    """Calculate the dust mass per species.
+
+    For dust/gas mixtures this is from the dust fraction.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    ndarray
+        The dust mass per species on the particles.
+    """
+    if snap.properties['dust_method'] == 'dust/gas mixture':
+        dust_fraction: ndarray = snap['dust_fraction']
+
+    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
+        n_dust = len(snap.properties.get('grain_size', []))
+        dust_type = snap['dust_type']
+        dust_fraction = np.zeros((len(snap), n_dust))
+        for idx in range(1, n_dust + 1):
+            dust_fraction[dust_type == idx, idx - 1] = 1
+
+    else:
+        raise ValueError('No dust available')
+
+    mass: ndarray = snap['mass']
+    if ignore_accreted:
+        h: ndarray = snap['smoothing_length']
+        dust_mass = mass[:, np.newaxis] * dust_fraction
+        return dust_mass[h > 0]
+    return mass[:, np.newaxis] * dust_fraction
 
 
 def dust_density(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
-    """Calculate the dust density per species from the dust fraction.
+    """Calculate the dust density per species.
+
+    For dust/gas mixtures this is from the dust fraction.
 
     Parameters
     ----------
@@ -424,15 +634,25 @@ def dust_density(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     ndarray
         The dust density per species on the particles.
     """
-    if ignore_accreted:
-        h: ndarray = snap['smooth']
-        dustfrac: ndarray = snap['dustfrac'][h > 0]
-        density: ndarray = snap['density'][h > 0]
-    else:
-        dustfrac = snap['dustfrac']
-        density = snap['density']
+    if snap.properties['dust_method'] == 'dust/gas mixture':
+        dust_fraction: ndarray = snap['dust_fraction']
 
-    return dustfrac * density[:, np.newaxis]
+    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
+        n_dust = len(snap.properties.get('grain_size', []))
+        dust_type = snap['dust_type']
+        dust_fraction = np.zeros((len(snap), n_dust))
+        for idx in range(1, n_dust + 1):
+            dust_fraction[dust_type == idx, idx - 1] = 1
+
+    else:
+        raise ValueError('No dust available')
+
+    density: ndarray = snap['density']
+    if ignore_accreted:
+        h: ndarray = snap['smoothing_length']
+        dust_mass = density[:, np.newaxis] * dust_fraction
+        return dust_mass[h > 0]
+    return density[:, np.newaxis] * dust_fraction
 
 
 def radial_distance(
@@ -458,7 +678,7 @@ def radial_distance(
         The radial distance on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         pos: ndarray = snap['position'][h > 0]
     else:
         pos = snap['position']
@@ -489,7 +709,7 @@ def azimuthal_angle(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The azimuthal angle on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         pos: ndarray = snap['position'][h > 0]
     else:
         pos = snap['position']
@@ -514,7 +734,7 @@ def polar_angle(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The azimuthal angle on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         pos: ndarray = snap['position'][h > 0]
     else:
         pos = snap['position']
@@ -547,7 +767,7 @@ def radial_velocity(
         The radial velocity on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         pos: ndarray = snap['position'][h > 0]
         vel: ndarray = snap['velocity'][h > 0]
     else:
@@ -583,7 +803,7 @@ def angular_velocity(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The angular velocity on the particles.
     """
     if ignore_accreted:
-        h: ndarray = snap['smooth']
+        h: ndarray = snap['smoothing_length']
         pos: ndarray = snap['position'][h > 0]
         vel: ndarray = snap['velocity'][h > 0]
     else:
@@ -596,3 +816,44 @@ def angular_velocity(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     vphi = (x * vy - y * vx) / (x ** 2 + y ** 2)
 
     return vphi
+
+
+def temperature(
+    snap: SnapLike, molecular_weight: float = 2.381, ignore_accreted: bool = False
+) -> ndarray:
+    """Calculate the gas temperature.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    molecular_weight
+        The gas molecular weight in gram / mole. E.g. 2.381 for
+        molecular hydrogen.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    ndarray
+        The gas temperature on the particles.
+    """
+    if ignore_accreted:
+        h: ndarray = snap['smoothing_length']
+        cs: ndarray = snap['sound_speed'][h > 0]
+    else:
+        cs = snap['sound_speed']
+
+    gamma = snap.properties['adiabatic_index']
+
+    molecular_weight = molecular_weight * plonk_units('gram / mole')
+    specific_gas_constant = (plonk_units.R / molecular_weight).to_base_units()
+
+    if isinstance(cs, Quantity):
+        T = cs ** 2 / (gamma * specific_gas_constant)
+    else:
+        T = (cs * snap.units['velocity']) ** 2 / (gamma * specific_gas_constant)
+
+    if isinstance(cs, Quantity):
+        return T
+    return T.magnitude
