@@ -4,6 +4,7 @@ Heavily inspired by pynbody (https://pynbody.github.io/).
 """
 
 from bisect import bisect
+from functools import partial
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 import matplotlib.pyplot as plt
@@ -13,10 +14,11 @@ from numpy import ndarray
 from pandas import DataFrame
 
 from .. import Quantity
-from ..snap import Snap
-from ..utils.math import norm
-from .particles import eccentricity as _eccentricity
-from .particles import specific_angular_momentum
+from .. import units as plonk_units
+from ..snap import SnapLike, gravitational_constant_in_code_units
+from ..utils import average, is_documented_by
+
+_aggregations = ('average', 'mean', 'median', 'std', 'sum')
 
 
 class Profile:
@@ -43,6 +45,10 @@ class Profile:
         distance.
     n_bins : optional
         The number of radial bins. Default is 100.
+    aggregation : optional
+        The method to aggregate particle quantities in bins by. Options
+        are 'average', 'mean', or 'median'. Here 'average' is a
+        mass-weighted average. Default is 'average'.
     spacing : optional
         The spacing of radial bins. Can be 'linear' or 'log'. Default is
         'linear'.
@@ -53,25 +59,23 @@ class Profile:
     --------
     Generate profile from snapshot.
 
-    >>> prof = plonk.Profile(snap=snap)
-    >>> prof = plonk.Profile(snap=snap, n_bins=300)
-    >>> prof = plonk.Profile(snap=snap, radius_min=1, radius_max=300)
-    >>> prof = plonk.Profile(snap=snap, spacing='log')
+    >>> prof = plonk.load_profile(snap=snap)
+    >>> prof = plonk.load_profile(snap=snap, n_bins=300)
+    >>> prof = plonk.load_profile(snap=snap, radius_min=1, radius_max=300)
+    >>> prof = plonk.load_profile(snap=snap, spacing='log')
 
     If snap has physical units and setting 'radius_min' or 'radius_max'
     must use physical quantities.
 
     >>> radius_min = plonk.Quantity('1 au')
     >>> radius_max = plonk.Quantity('300 au')
-    >>> prof = plonk.Profile(
-    ...     snap=snap,
-    ...     radius_min=radius_min,
-    ...     radius_max=radius_max
+    >>> prof = plonk.load_profile(
+    ...     snap=snap, radius_min=radius_min, radius_max=radius_max,
     ... )
 
     To access a profile.
 
-    >>> prof['density']
+    >>> prof['surface_density']
     >>> prof['scale_height']
 
     To set a new profile.
@@ -83,16 +87,20 @@ class Profile:
     >>> @Profile.profile_property
     ... def mass(prof):
     ...     M = prof.snap['mass']
-    ...     return prof.particles_to_binned_quantity(np.sum, M)
+    ...     return prof.particles_to_binned_quantity('sum', M)
 
     Plot one or many quantities on the profile.
 
     >>> prof.plot('radius', 'density')
-    >>> prof.plot('radius', ['angmom_x', angmom_y'])
+    >>> prof.plot(
+    ...     'radius', ['angular_momentum_x', 'angular_momentum_y'],
+    ... )
 
     Plot a quantity on the profile with units.
 
-    >>> prof.plot('radius', 'density', x_unit='au', y_unit='g/cm**2')
+    >>> prof.plot(
+    ...     'radius', 'surface_density', x_unit='au', y_unit='g/cm^2'
+    ... )
     """
 
     _profile_functions: Dict[str, Callable] = {}
@@ -117,22 +125,25 @@ class Profile:
 
     def __init__(
         self,
-        snap: Snap,
+        snap: SnapLike,
         ndim: Optional[int] = 2,
         radius_min: Optional[Any] = None,
         radius_max: Optional[Any] = None,
         n_bins: int = 100,
+        aggregation: str = 'average',
         spacing: str = 'linear',
         ignore_accreted: bool = True,
     ):
 
         self.snap = snap
         self.ndim = ndim
+        self.aggregation = self._check_aggregation(aggregation)
         self.spacing = self._check_spacing(spacing)
         self.properties: Dict[str, Any] = {}
 
         self._profiles: Dict[str, ndarray] = {}
 
+        self._weights = self.snap['mass']
         self._mask = self._setup_particle_mask(ignore_accreted)
         self._x = self._calculate_x()
         self.range = self._set_range(radius_min, radius_max)
@@ -156,14 +167,22 @@ class Profile:
         else:
             self._profiles['number'] = np.histogram(self._x, self.bin_edges)[0]
 
-    def _check_spacing(self, spacing):
+        n_dust = len(self.snap.properties.get('grain_size', []))
+        _generate_profiles(n_dust)
+
+    def _check_aggregation(self, method: str) -> str:
+        if method in _aggregations:
+            return method
+        raise ValueError(
+            f'Cannot determine aggregation method: choose from {_aggregations}'
+        )
+
+    def _check_spacing(self, spacing: str) -> str:
         if spacing.lower() in ('lin', 'linear'):
-            spacing = 'linear'
+            return 'linear'
         elif spacing.lower() in ('log', 'logarithm', 'logarithmic'):
-            spacing = 'log'
-        else:
-            raise ValueError('Cannot determine spacing')
-        return spacing
+            return 'log'
+        raise ValueError('Cannot determine spacing')
 
     def _setup_particle_mask(self, ignore_accreted: bool) -> ndarray:
         if ignore_accreted is False:
@@ -207,19 +226,21 @@ class Profile:
         if radius_min is None:
             rmin = self._x.min()
         else:
-            if not isinstance(radius_min, Quantity):
+            rmin = Quantity(radius_min)
+            if not rmin.dimensionality == Quantity('cm').dimensionality:
                 raise ValueError(
                     'Snap has physical units: must use dimensional radius_min'
                 )
-            rmin = radius_min.to_base_units()
+            rmin = rmin.to_base_units()
         if radius_max is None:
             rmax = np.percentile(self._x.magnitude, 99, axis=0) * self._x.units
         else:
-            if not isinstance(radius_max, Quantity):
+            rmax = Quantity(radius_max)
+            if not rmax.dimensionality == Quantity('cm').dimensionality:
                 raise ValueError(
                     'Snap has physical units: must use dimensional radius_max'
                 )
-            rmax = radius_max.to_base_units()
+            rmax = rmax.to_base_units()
 
         return rmin, rmax
 
@@ -277,24 +298,48 @@ class Profile:
             prev_index = new_index
         return binind
 
-    def _get_profile(self, name: str, args: Optional[Tuple[Any, ...]] = None):
-        """Get a profile by name."""
+    def _getitem(self, name: str) -> ndarray:
+        """Return the profile of a given kind."""
+        name_root = '_'.join(name.split('_')[:-1])
+        name_suffix = name.split('_')[-1]
+
         if name in self._profiles:
             return self._profiles[name]
-
         elif name in Profile._profile_functions:
-            if args is not None:
-                self._profiles[name] = Profile._profile_functions[name](self, *args)
-            else:
-                self._profiles[name] = Profile._profile_functions[name](self)
+            self._profiles[name] = Profile._profile_functions[name](self)
             return self._profiles[name]
 
+        if name_suffix in _aggregations:
+            aggregation = name_suffix
+            array_name = name_root
         else:
-            raise ValueError('Profile not available')
+            aggregation = self.aggregation
+            array_name = name
+        try:
+            array: ndarray = self.snap[array_name]
+        except ValueError:
+            if self.snap._extra_quantities:
+                print('Profile unavailable.')
+            else:
+                print(
+                    'Profile unavailable. Try calling extra_quantities method on Snap.'
+                )
+            return
+        if array.ndim == 1:
+            self._profiles[name] = self.particles_to_binned_quantity(aggregation, array)
+            return self._profiles[name]
+        else:
+            raise ValueError(
+                'Requested profile has array dimension > 1.\nTo access x-, y-, or '
+                'z-components, or magnitude of vector quantities,\ntry, for '
+                'example, prof["velocity_x"] or prof["momentum_magnitude"].\nTo '
+                'access dust profiles, try, for example, prof["stopping_time_001"] '
+                'or\nprof["dust_mass_sum"].'
+            )
 
     def __getitem__(self, name: str) -> ndarray:
         """Return the profile of a given kind."""
-        return self._get_profile(name)
+        return self._getitem(name)
 
     def __setitem__(self, name: str, item: ndarray):
         """Set the profile directly."""
@@ -302,15 +347,15 @@ class Profile:
             raise ValueError('"item" must be ndarray or pint Quantity')
         if item.shape[0] != self.n_bins:
             raise ValueError('Length of array does not match number of bins')
-        if name in self.loaded_keys():
+        if name in self.loaded_profiles():
             raise ValueError(
                 'Attempting to overwrite existing profile. To do so, first delete the '
                 'profile\nwith del prof["profile"], then try again.'
             )
-        elif name in self.available_keys():
+        elif name in self.available_profiles():
             raise ValueError(
                 'Attempting to set profile already available. '
-                'See prof.available_keys().'
+                'See prof.available_profiles().'
             )
         self._profiles[name] = item
 
@@ -323,18 +368,23 @@ class Profile:
         return self.n_bins
 
     def __repr__(self):
-        """Object repr method."""
-        return f'<plonk.Profile: {self.n_bins} bins>'
+        """Dunder repr method."""
+        return self.__str__()
 
-    def loaded_keys(self):
+    def __str__(self):
+        """Dunder str method."""
+        return f'<plonk.Profile "{self.snap.file_path.name}">'
+
+    def loaded_profiles(self):
         """Return a listing of loaded profiles."""
         return tuple(sorted(self._profiles.keys()))
 
-    def available_keys(self):
+    def available_profiles(self):
         """Return a listing of available profiles."""
-        loaded = list(self.loaded_keys())
+        loaded = list(self.loaded_profiles())
         available = list(self._profile_functions.keys())
-        return tuple(sorted(set(loaded + available)))
+        snap_arrays = list(self.snap.available_arrays())
+        return tuple(sorted(set(loaded + available + snap_arrays)))
 
     def plot(
         self,
@@ -342,7 +392,9 @@ class Profile:
         y: Union[str, List[str]],
         x_unit: Optional[str] = None,
         y_unit: Optional[Union[str, List[str]]] = None,
+        std_dev_shading: bool = False,
         ax: Any = None,
+        **kwargs,
     ):
         """Plot profile.
 
@@ -359,12 +411,13 @@ class Profile:
         y_unit : optional
             The y axis quantity unit as a string or list of strings.
             Only works if using physical units.
+        std_dev_shading : optional
+            Add shading for standard deviation of profile.
         ax : optional
             A matplotlib Axes object to plot to.
+        **kwargs
+            Keyword arguments to pass to Axes plot method.
         """
-        if x not in self.available_keys():
-            raise ValueError('Cannot determine x axis to plot')
-
         if isinstance(y, str):
             y = [y]
 
@@ -380,7 +433,10 @@ class Profile:
 
         _x = self[x]
         if x_unit is not None:
-            _x = _x.to(x_unit)
+            if not self.snap._physical_units:
+                raise ValueError('Cannot set unit if snap is not in physical units')
+            else:
+                _x = _x.to(x_unit)
 
         if ax is None:
             fig, ax = plt.subplots()
@@ -390,44 +446,67 @@ class Profile:
         xlabel = x.capitalize().replace('_', ' ')
         if self.snap._physical_units:
             xlabel = ' '.join([xlabel, f'[{_x.units:~P}]'])
+            _x = _x.magnitude
         ax.set_xlabel(xlabel)
 
         for idx, yi in enumerate(y):
-            if yi not in self.available_keys():
-                raise ValueError(f'Cannot determine y axis to plot: {yi} not available')
             _y = self[yi]
+            if std_dev_shading:
+                if yi.split('_')[-1] in _aggregations:
+                    _yi = '_'.join(yi.split('_')[:-1])
+                else:
+                    _yi = yi
+                _y_std = self[_yi + '_std']
+                if self.aggregation != 'mean':
+                    _y_mean = self[_yi + '_mean']
+                else:
+                    _y_mean = _y
             label = yi.capitalize().replace('_', ' ')
             if self.snap._physical_units:
                 if y_unit is not None:
                     _y = _y.to(y_unit[idx])
+                    if std_dev_shading:
+                        _y_std = _y_std.to(y_unit[idx])
+                        _y_mean = _y_mean.to(y_unit[idx])
                 label = ' '.join([label, f'[{_y.units:~P}]'])
-                ax.plot(_x.magnitude, _y.magnitude, label=label)
-            else:
-                ax.plot(_x, _y, label=label)
+                _y = _y.magnitude
+                if std_dev_shading:
+                    _y_std = _y_std.magnitude
+                    _y_mean = _y_mean.magnitude
+            ax.plot(_x, _y, label=label, **kwargs)
+            if std_dev_shading:
+                color = ax.lines[idx].get_color()
+                ax.fill_between(
+                    _x, _y_mean - _y_std, _y_mean + _y_std, color=color, alpha=0.2,
+                )
 
         ax.legend()
 
         return fig, ax
 
-    def to_dataframe(self, all_available: bool = False) -> DataFrame:
+    def to_dataframe(
+        self, columns: Union[Tuple[str, ...], List[str]] = None
+    ) -> DataFrame:
         """Convert Profile to DataFrame.
 
         Parameters
         ----------
-        all_available
-            If True, this will calculate all available profiles before
-            converting to a DataFrame.
+        columns : optional
+            A list of columns to add to the data frame. Default is
+            None.
+
+        Returns
+        -------
+        DataFrame
         """
-        if all_available:
-            columns = self.available_keys()
-        else:
-            columns = self.loaded_keys()
         data = dict()
+        if columns is None:
+            columns = self.loaded_profiles()
         for column in columns:
             data[column] = self[column]
         return pd.DataFrame(data)
 
-    def particles_to_binned_quantity(self, function, *args) -> ndarray:
+    def particles_to_binned_quantity(self, aggregation: str, array: ndarray) -> ndarray:
         """Calculate binned quantities from particles.
 
         This takes care of the bin indices and ignoring accreted
@@ -435,133 +514,157 @@ class Profile:
 
         Parameters
         ----------
-        function
-            The function that acts on particles in the radial bin. The
-            function should produce a single number, i.e. it is a
-            reduction. E.g. to calculate the mean of a particle quantity
-            in a radial bin use np.mean.
-        *args
-            The arguments to the function. Typically this is one or more
-            particle arrays or other quantities on a snapshot.
+        aggregation
+            The aggregation function that acts on particles in the
+            radial bin.
+        array
+            The particle array.
         """
+        if aggregation not in _aggregations:
+            raise ValueError('Cannot determine aggregation method')
+
         binned_quantity = np.zeros(self.n_bins)
         for idx, bin_ind in enumerate(self.bin_indicies):
-            val = function(*tuple([arg[self._mask][bin_ind] for arg in args]))
+            if bin_ind.size == 0:
+                continue
+            if aggregation == 'average':
+                val = average(
+                    array[self._mask][bin_ind],
+                    weights=self._weights[self._mask][bin_ind],
+                )
+            elif aggregation == 'mean':
+                val = np.mean(array[self._mask][bin_ind])
+            elif aggregation == 'median':
+                val = np.median(array[self._mask][bin_ind])
+            elif aggregation == 'std':
+                val = np.std(array[self._mask][bin_ind])
+            elif aggregation == 'sum':
+                val = np.sum(array[self._mask][bin_ind])
+
             if self.snap._physical_units:
                 binned_quantity[idx] = val.magnitude
             else:
                 binned_quantity[idx] = val
+
         if self.snap._physical_units:
             binned_quantity *= val.units
+
         return binned_quantity
 
 
-@Profile.profile_property
-def mass(prof) -> ndarray:
-    """Mass profile."""
-    M = prof.snap['mass']
-    return prof.particles_to_binned_quantity(np.sum, M)
+def _generate_profiles(n_dust: int = 0):
+    @Profile.profile_property
+    def mass(prof) -> ndarray:
+        """Mass profile."""
+        M = prof.snap['mass']
+        return prof.particles_to_binned_quantity('sum', M)
 
+    @Profile.profile_property
+    def surface_density(prof) -> ndarray:
+        """Surface density profile.
 
-@Profile.profile_property
-def density(prof) -> ndarray:
-    """Density profile.
+        Units are [mass / length ** ndim], which depends on ndim of profile.
+        """
+        return prof['mass'] / prof['size']
 
-    Units are [mass / length ** ndim], which depends on ndim of profile.
-    """
-    return prof['mass'] / prof['size']
+    @Profile.profile_property
+    def scale_height(prof) -> ndarray:
+        """Scale height profile."""
+        z = prof.snap['z']
+        return prof.particles_to_binned_quantity('std', z)
 
+    @Profile.profile_property
+    def aspect_ratio(prof) -> ndarray:
+        """Aspect ratio profile."""
+        H = prof['scale_height']
+        R = prof['radius']
+        return H / R
 
-@Profile.profile_property
-def smooth(prof) -> ndarray:
-    """Smoothing length profile."""
-    h = prof.snap['smooth']
-    return prof.particles_to_binned_quantity(np.mean, h)
+    @Profile.profile_property
+    def angular_momentum_theta(prof) -> ndarray:
+        """Angle between specific angular momentum and xy-plane."""
+        angular_momentum_z = prof['angular_momentum_z']
+        angular_momentum_magnitude = prof['angular_momentum_magnitude']
 
+        return np.arccos(angular_momentum_z / angular_momentum_magnitude)
 
-@Profile.profile_property
-def scale_height(prof) -> ndarray:
-    """Scale height profile."""
-    z = prof.snap['z']
-    return prof.particles_to_binned_quantity(np.std, z)
+    @Profile.profile_property
+    def angular_momentum_phi(prof) -> ndarray:
+        """Angle between specific angular momentum and x-axis in xy-plane."""
+        angular_momentum_x = prof['angular_momentum_x']
+        angular_momentum_y = prof['angular_momentum_y']
+        return np.arctan2(angular_momentum_y, angular_momentum_x)
 
-
-@Profile.profile_property
-def aspect_ratio(prof) -> ndarray:
-    """Aspect ratio profile."""
-    H = prof['scale_height']
-    R = prof['radius']
-    return H / R
-
-
-@Profile.profile_property
-def angmom_mag(prof) -> ndarray:
-    """Magnitude of specific angular momentum profile."""
-    angmom = specific_angular_momentum(snap=prof.snap)
-    angmom_mag = norm(angmom, axis=1)
-
-    return prof.particles_to_binned_quantity(np.mean, angmom_mag)
-
-
-@Profile.profile_property
-def angmom_x(prof) -> ndarray:
-    """x-component of specific angular momentum profile."""
-    angmom = specific_angular_momentum(snap=prof.snap)
-    angmom_x = angmom[:, 0]
-
-    return prof.particles_to_binned_quantity(np.mean, angmom_x)
-
-
-@Profile.profile_property
-def angmom_y(prof) -> ndarray:
-    """y-component of specific angular momentum profile."""
-    angmom = specific_angular_momentum(snap=prof.snap)
-    angmom_y = angmom[:, 1]
-
-    return prof.particles_to_binned_quantity(np.mean, angmom_y)
-
-
-@Profile.profile_property
-def angmom_z(prof) -> ndarray:
-    """z-component of specific angular momentum profile."""
-    angmom = specific_angular_momentum(snap=prof.snap)
-    angmom_z = angmom[:, 2]
-
-    return prof.particles_to_binned_quantity(np.mean, angmom_z)
-
-
-@Profile.profile_property
-def angmom_theta(prof) -> ndarray:
-    """Angle between specific angular momentum and xy-plane."""
-    angmom_z = prof['angmom_z']
-    angmom_mag = prof['angmom_mag']
-
-    return np.arccos(angmom_z / angmom_mag)
-
-
-@Profile.profile_property
-def angmom_phi(prof) -> ndarray:
-    """Angle between specific angular momentum and x-axis in xy-plane."""
-    angmom_x = prof['angmom_x']
-    angmom_y = prof['angmom_y']
-    return np.arctan2(angmom_y, angmom_x)
-
-
-@Profile.profile_property
-def eccentricity(prof) -> ndarray:
-    """Orbital eccentricity profile.
-
-    This profile assumes the central mass is at (0, 0, 0) and that the
-    gravitational parameter (mu = G M) is set.
-    """
-    try:
-        gravitational_parameter = prof.properties['gravitational_parameter']
-    except KeyError:
-        raise ValueError(
-            'To generate eccentricity profile, first set the gravitational parameter\n'
-            'via prof.properties["gravitational_parameter"].'
+    @Profile.profile_property
+    def toomre_Q(prof) -> ndarray:
+        """Toomre Q parameter."""
+        if not prof.snap._physical_units:
+            G = gravitational_constant_in_code_units(prof.snap)
+        else:
+            G = (1 * plonk_units.newtonian_constant_of_gravitation).to_base_units()
+        return (
+            prof['sound_speed']
+            * prof['keplerian_frequency']
+            / (np.pi * G * prof['density'])
         )
-    ecc = _eccentricity(
-        snap=prof.snap, gravitational_parameter=gravitational_parameter,
+
+    if n_dust > 0:
+
+        @Profile.profile_property
+        def gas_mass(prof) -> ndarray:
+            """Gas mass profile."""
+            M = prof.snap['gas_mass']
+            return prof.particles_to_binned_quantity('sum', M)
+
+        @Profile.profile_property
+        def gas_surface_density(prof) -> ndarray:
+            """Gas surface density profile.
+
+            Units are [mass / length ** ndim], which depends on ndim of profile.
+            """
+            return prof['gas_mass'] / prof['size']
+
+        for idx in range(n_dust):
+
+            def dust_mass(idx, prof) -> ndarray:
+                """Dust mass profile."""
+                M = prof.snap[f'dust_mass_{idx+1:03}']
+                return prof.particles_to_binned_quantity('sum', M)
+
+            def dust_surface_density(idx, prof) -> ndarray:
+                """Dust surface density profile.
+
+                Units are [mass / length ** ndim], which depends on ndim of profile.
+                """
+                return prof[f'dust_mass_{idx+1:03}'] / prof['size']
+
+            Profile._profile_functions[f'dust_mass_{idx+1:03}'] = partial(
+                dust_mass, idx
+            )
+
+            Profile._profile_functions[f'dust_surface_density_{idx+1:03}'] = partial(
+                dust_surface_density, idx
+            )
+
+
+@is_documented_by(Profile)
+def load_profile(
+    snap: SnapLike,
+    ndim: Optional[int] = 2,
+    radius_min: Optional[Any] = None,
+    radius_max: Optional[Any] = None,
+    n_bins: int = 100,
+    aggregation: str = 'average',
+    spacing: str = 'linear',
+    ignore_accreted: bool = True,
+) -> Profile:
+    return Profile(
+        snap=snap,
+        ndim=ndim,
+        radius_min=radius_min,
+        radius_max=radius_max,
+        n_bins=n_bins,
+        aggregation=aggregation,
+        spacing=spacing,
+        ignore_accreted=ignore_accreted,
     )
-    return prof.particles_to_binned_quantity(np.mean, ecc)
