@@ -1,10 +1,11 @@
 """The utils sub-package."""
 
 from .geometry import cartesian_to_polar
-from .math import cross, norm
+from .math import average, cross, norm
 from .utils import get_extent_from_percentile, is_documented_by, time_string
 
 __all__ = [
+    'average',
     'cartesian_to_polar',
     'cross',
     'is_documented_by',
