@@ -5,14 +5,16 @@ simulations using kernel density estimation based interpolation.
 """
 
 from .animation import animation, animation_profiles
+from .functions import particle_plot, plot, plot_snaps, str_to_units
 from .interpolation import interpolate
-from .visualization import Visualization, plot, plot_snaps, str_to_units
+from .visualization import Visualization
 
 __all__ = [
     'Visualization',
     'animation',
     'animation_profiles',
     'interpolate',
+    'particle_plot',
     'plot',
     'plot_snaps',
     'str_to_units',
