@@ -6,18 +6,16 @@ hydrodynamics simulation data.
 
 from __future__ import annotations
 
-from typing import Any, Dict, List, Optional, Union
+from copy import copy
+from typing import Any, Dict, Optional, Union
 
 import matplotlib.pyplot as plt
-import numpy as np
 from mpl_toolkits.axes_grid1 import make_axes_locatable
 from numpy import ndarray
 
-from .. import Quantity
-from .. import units as plonk_units
 from ..snap import SnapLike
 from ..snap.snap import get_array_from_input
-from ..utils import get_extent_from_percentile, is_documented_by
+from ..utils import get_extent_from_percentile
 from . import plots
 from .interpolation import Extent, interpolate
 
@@ -64,6 +62,7 @@ class Visualization:
         A dictionary containing the matplotlib plot objects:
 
         - 'lines' : list of matplotlib Line2D objects for particle plots
+        - 'paths' : matplotlib PathCollection object for scatter plots
         - 'image' : matplotlib AxesImage object for rendered plots
         - 'colorbar' : matplotlib Colorbar object for rendered plots
         - 'contour' : matplotlib QuadContourSet object for contour plots
@@ -86,6 +85,7 @@ class Visualization:
         self.extent: Extent = (-1, -1, -1, -1)
         self.objects: Dict[str, Any] = {
             'lines': None,
+            'paths': None,
             'image': None,
             'colorbar': None,
             'contour': None,
@@ -102,7 +102,7 @@ class Visualization:
     def plot(
         self,
         *,
-        quantity: Optional[Union[str, ndarray]] = None,
+        quantity: Union[str, ndarray],
         x: Union[str, ndarray] = 'x',
         y: Union[str, ndarray] = 'y',
         z: Union[str, ndarray] = 'z',
@@ -110,15 +110,15 @@ class Visualization:
         interp: str = 'projection',
         z_slice: float = 0.0,
         extent: Extent = (-1, -1, -1, -1),
-        units=None,
+        units: Dict[str, Any] = None,
         ax: Optional[Any] = None,
         **kwargs,
     ) -> Visualization:
         """Visualize smoothed particle hydrodynamics data.
 
-        Visualize SPH data by showing the particle positions, a rendered
-        image or a contour plot for scalar data, a quiver (arrow) plot
-        or stream plot for vector data.
+        Visualize SPH data by interpolation to a pixel grid. Including:
+        a rendered image or a contour plot for scalar data, a quiver
+        (arrow) plot or stream plot for vector data.
 
         Parameters
         ----------
@@ -127,7 +127,7 @@ class Visualization:
             or a 1d array (N,) of scalar data, or a 2d array (N, 3) of
             vector data. If quantity is 2d, only the first two
             components are visualized, i.e. quantity[:, 0] and
-            quantity[:, 1]. Default is None.
+            quantity[:, 1].
         x
             The x-coordinate for the visualization. Can be a string to
             pass to Snap, or a 1d array (N,). Default is 'x'.
@@ -141,11 +141,9 @@ class Visualization:
         kind
             The type of plot.
 
-            - 'particle' : particle plot (default if quantity is None)
             - 'render' : rendered image (default for scalar quantities)
             - 'contour' : contour plot (scalar quantity)
-            - 'quiver' : quiver (arrow) plot (default for vector
-              quantities)
+            - 'quiver' : quiver plot (default for vector quantities)
             - 'stream' : stream plot (vector quantity)
         interp
             The interpolation type. Default is 'projection'.
@@ -202,10 +200,6 @@ class Visualization:
 
         Examples
         --------
-        Plot the particles.
-
-        >>> viz = plonk.visualize.plot(snap=snap)
-
         Render the surface density in xy-plane.
 
         >>> viz = plonk.visualize.plot(
@@ -252,6 +246,8 @@ class Visualization:
 
         >>> units = plonk.visualize.str_to_units('g/cm^3', 'au', 'cm')
         """
+        _kwargs = copy(kwargs)
+
         if self.ax is None:
             if ax is None:
                 self.fig, self.ax = plt.subplots()
@@ -262,65 +258,60 @@ class Visualization:
             if ax is not None:
                 raise ValueError('Trying to change existing Axes attribute')
 
-        if extent == (-1, -1, -1, -1):
-            extent = get_extent_from_percentile(self.snap)
-
         quantity_str: Optional[str] = quantity if isinstance(quantity, str) else None
         quantity, x, y, z, kind = _check_input(
             snap=self.snap, quantity=quantity, x=x, y=y, z=z, kind=kind
         )
 
-        if quantity is not None:
-            interpolation_kwargs = ('number_of_pixels', 'density_weighted')
-            _kwargs = {
-                key: val for key, val in kwargs.items() if key in interpolation_kwargs
-            }
-            for key in _kwargs:
-                kwargs.pop(key)
-            interpolated_data = interpolate(
-                snap=self.snap,
-                quantity=quantity,
-                x=x,
-                y=y,
-                z=z,
-                interp=interp,
-                z_slice=z_slice,
-                extent=extent,
-                **_kwargs,
-            )
-            if units is not None:
-                if quantity_str is None:
-                    raise ValueError(
-                        'Cannot set units when passing in arrays. '
-                        'Instead, use strings to access\n'
-                        'quantities on Snap. E.g. plot(..., quantity="density", ...).'
-                    )
-                interpolated_data, extent = _convert_units(
-                    snap=self.snap,
-                    quantity_str=quantity_str,
-                    interpolated_data=interpolated_data,
-                    extent=extent,
-                    units=units,
-                    interp=interp,
+        if extent == (-1, -1, -1, -1):
+            extent = get_extent_from_percentile(x, y)
+        interpolation_kwargs = ('number_of_pixels', 'density_weighted')
+        __kwargs = {
+            key: val for key, val in _kwargs.items() if key in interpolation_kwargs
+        }
+        for key in __kwargs:
+            _kwargs.pop(key)
+        interpolated_data = interpolate(
+            snap=self.snap,
+            quantity=quantity,
+            x=x,
+            y=y,
+            z=z,
+            interp=interp,
+            z_slice=z_slice,
+            extent=extent,
+            **__kwargs,
+        )
+        if units is not None:
+            if quantity_str is None:
+                raise ValueError(
+                    'Cannot set units when passing in arrays. '
+                    'Instead, use strings to access\n'
+                    'quantities on Snap. E.g. plot(..., quantity="density", ...).'
                 )
-                self.units.update(units)
+            interpolated_data, extent = _convert_units(
+                snap=self.snap,
+                quantity_str=quantity_str,
+                interpolated_data=interpolated_data,
+                extent=extent,
+                units=units,
+                interp=interp,
+            )
+            self.units.update(units)
 
         self.extent = extent
 
-        show_colorbar = kwargs.pop('show_colorbar', True if kind == 'render' else False)
+        show_colorbar = _kwargs.pop(
+            'show_colorbar', True if kind == 'render' else False
+        )
 
-        if kind == 'particle':
-            self.objects['lines'] = plots.particle_plot(
-                snap=self.snap, x=x, y=y, extent=extent, ax=self.ax, **kwargs,
-            )
-
-        elif kind in ('render', 'contour', 'quiver', 'stream'):
+        if kind in ('render', 'contour', 'quiver', 'stream'):
             self.data[kind] = interpolated_data
             self.objects[_kind_to_object[kind]] = _kind_to_function[kind](
                 interpolated_data=interpolated_data,
                 extent=extent,
                 ax=self.ax,
-                **kwargs,
+                **_kwargs,
             )
 
         else:
@@ -333,95 +324,109 @@ class Visualization:
 
         self.ax.set_xlim(*extent[:2])
         self.ax.set_ylim(*extent[2:])
-        self.ax.set_aspect('equal')
+
+        ratio = (extent[1] - extent[0]) / (extent[3] - extent[2])
+        if not max(ratio, 1 / ratio) > 10.0:
+            self.ax.set_aspect('equal')
+
+        return self
+
+    def particle_plot(
+        self,
+        *,
+        x: Union[str, ndarray] = 'x',
+        y: Union[str, ndarray] = 'y',
+        color: Optional[Union[str, ndarray]] = None,
+        size: Optional[Union[str, ndarray]] = None,
+        units: Dict[str, Any] = None,
+        xscale: str = None,
+        yscale: str = None,
+        ax: Optional[Any] = None,
+        **kwargs,
+    ) -> Visualization:
+        """Visualize smoothed particle hydrodynamics data.
+
+        Visualize SPH data by plotting the particles.
+
+        Parameters
+        ----------
+        x
+            The x-coordinate for the visualization. Can be a string to
+            pass to Snap, or a 1d array (N,). Default is 'x'.
+        y
+            The y-coordinate for the visualization. Can be a string to
+            pass to Snap, or a 1d array (N,). Default is 'y'.
+        color
+            The quantity to color the particles.
+        size
+            The quantity to set the particle size.
+        units
+            The units of the plot as a dictionary with keys 'quantity',
+            'extent', 'projection'. The values are Pint Unit objects.
+        xscale
+            The xscale to pass to the matplotlib Axes method set_xscale.
+        yscale
+            The yscale to pass to the matplotlib Axes method set_yscale.
+        ax
+            A matplotlib Axes handle.
+        **kwargs
+            Additional keyword arguments to pass to matplotlib
+            functions.
+        """
+        _kwargs = copy(kwargs)
+
+        if self.ax is None:
+            if ax is None:
+                self.fig, self.ax = plt.subplots()
+            else:
+                self.fig = ax.get_figure()
+                self.ax = ax
+        else:
+            if ax is not None:
+                raise ValueError('Trying to change existing Axes attribute')
+
+        x, y, color, size = _check_input_particles(
+            snap=self.snap, x=x, y=y, color=color, size=size,
+        )
+
+        h: ndarray = self.snap['smoothing_length']
+        mask = h > 0
+
+        x = x[mask]
+        y = y[mask]
+        if color is not None:
+            color = color[mask]
+        if size is not None:
+            size = size[mask]
+
+        if size is None and color is None:
+            self.objects['lines'] = plots.particle_plot(
+                x=x, y=y, ax=self.ax, **_kwargs,
+            )
+
+        else:
+            self.objects['paths'] = plots.scatter_plot(
+                x=x, y=y, color=color, size=size, ax=self.ax, **_kwargs,
+            )
+
+        if xscale is not None:
+            self.ax.set_xscale(xscale)
+        if yscale is not None:
+            self.ax.set_yscale(yscale)
+
+        ratio = (x.max() - x.min()) / (y.max() - y.min())
+        if not max(ratio, 1 / ratio) > 10.0:
+            self.ax.set_aspect('equal')
 
         return self
 
     def __repr__(self):
         """Dunder repr method."""
-        return '<plonk.Visualization>'
+        return self.__str__()
 
-
-class MultiVisualization:
-    """Visualize multiple snaps.
-
-    Attributes
-    ----------
-    snaps
-        A list of snaps.
-    options
-        A dictionary of arguments passed to Visualization plot method.
-    visualization
-        The Visualization object.
-    ax
-        The matplotlib Axes object.
-
-    Parameters
-    ----------
-    snaps
-        A list of Snap objects.
-    **kwargs
-        Keyword arguments to pass to Visualization plot method.
-    """
-
-    def __init__(self, snaps, **kwargs):
-        self.snaps = snaps
-        self.options = kwargs
-        self.visualization = Visualization(snap=snaps[0]).plot(**kwargs)
-        self.ax = self.visualization.ax
-
-        self._len = -1
-        self._where = 0
-
-    def _fn(self, idx):
-        self.ax.clear()
-        cbar = self.visualization.objects['colorbar']
-        if cbar is not None:
-            cbar.remove()
-        viz = Visualization(snap=self.snaps[idx]).plot(ax=self.ax, **self.options)
-        self.visualization = viz
-        return viz
-
-    def next(self, number: int = 1):
-        """Visualize next snap."""
-        idx = self._where + number
-        if idx < len(self):
-            self.visualization = self._fn(idx)
-            self._where += number
-        else:
-            print('Too far forward. Going to last snap.')
-            self.visualization = self._fn(len(self) - 1)
-            self._where = len(self) - 1
-
-    def prev(self, number: int = 1):
-        """Visualize previous snap."""
-        idx = self._where - number
-        if idx > 0:
-            self.visualization = self._fn(idx)
-            self._where -= number
-        else:
-            print('Too far back. Going to first snap.')
-            self.visualization = self._fn(0)
-            self._where = 0
-
-    def goto(self, idx):
-        """Visualize particular snap by index."""
-        if -len(self) < idx < len(self) - 1:
-            self.visualization = self._fn(idx)
-            self._where = np.mod(idx, len(self))
-        else:
-            raise ValueError('out of range')
-
-    @property
-    def index(self):
-        """Current snap index."""
-        return self._where
-
-    def __len__(self):
-        """Length as number of snaps."""
-        if self._len == -1:
-            self._len = len(self.snaps)
-        return self._len
+    def __str__(self):
+        """Dunder str method."""
+        return f'<plonk.Visualization "{self.snap.file_path.name}">'
 
 
 def _check_input(*, snap, quantity, x, y, z, kind):
@@ -451,9 +456,25 @@ def _check_input(*, snap, quantity, x, y, z, kind):
         else:
             raise ValueError(f'No quantity: can only do particle plot')
 
+    if kind not in ('render', 'contour', 'quiver', 'stream'):
+        raise ValueError('Cannot determine plot kind')
+
     return quantity, x, y, z, kind
 
 
+def _check_input_particles(*, snap, x, y, color, size):
+
+    if color is not None:
+        color = get_array_from_input(snap, color)
+    if size is not None:
+        size = get_array_from_input(snap, size)
+
+    x = get_array_from_input(snap, x, 'x')
+    y = get_array_from_input(snap, y, 'y')
+
+    return x, y, color, size
+
+
 def _convert_units(
     *,
     snap: SnapLike,
@@ -479,109 +500,3 @@ def _convert_units(
     new_extent = tuple((extent * snap.units['length']).to(units['extent']).magnitude)
 
     return data, new_extent
-
-
-@is_documented_by(Visualization.plot)
-def plot(
-    *,
-    snap: SnapLike,
-    quantity: Optional[Union[str, ndarray]] = None,
-    x: Union[str, ndarray] = 'x',
-    y: Union[str, ndarray] = 'y',
-    z: Union[str, ndarray] = 'z',
-    kind: Optional[str] = None,
-    interp: str = 'projection',
-    z_slice: float = 0.0,
-    extent: Extent = (-1, -1, -1, -1),
-    ax: Optional[Any] = None,
-    **kwargs,
-) -> Visualization:
-    viz = Visualization(snap)
-    viz.plot(
-        quantity=quantity,
-        x=x,
-        y=y,
-        z=z,
-        kind=kind,
-        interp=interp,
-        z_slice=z_slice,
-        extent=extent,
-        ax=ax,
-        **kwargs,
-    )
-    return viz
-
-
-def plot_snaps(snaps: List[SnapLike], **kwargs) -> MultiVisualization:
-    """Visualize multiple snaps.
-
-    Parameters
-    ----------
-    snaps
-        A list of Snap objects.
-    **kwargs
-        Keyword arguments to pass to Visualization plot method.
-
-    Returns
-    -------
-    MultiVisualization
-
-    Examples
-    --------
-    Initialize object passing in plotting parameters.
-
-    >>> vi = plot_snaps(
-    ...     snaps=sim.snaps,
-    ...     quantity='density',
-    ... )
-
-    Go forwards and backwards through snaps.
-
-    >>> vi.next()
-    >>> vi.prev()
-
-    Go to a particular snap, or skip ahead.
-
-    >>> vi.goto(10)
-    >>> vi.next(5)
-    """
-    return MultiVisualization(snaps, **kwargs)
-
-
-def str_to_units(quantity, extent, projection):
-    """Convert string to plonk units.
-
-    Parameters
-    ----------
-    quantity
-        The units string for the quantity.
-    extent
-        The units string for the plot extent.
-    projection
-        The units string for projection interpolation.
-    """
-    if isinstance(quantity, str):
-        quantity = plonk_units(quantity)
-    elif isinstance(quantity, Quantity):
-        pass
-    else:
-        raise ValueError(f'Cannot determine quantity unit')
-    if isinstance(extent, str):
-        extent = plonk_units(extent)
-    elif isinstance(extent, Quantity):
-        pass
-    else:
-        raise ValueError(f'Cannot determine extent unit')
-    if isinstance(projection, str):
-        projection = plonk_units(projection)
-    elif isinstance(projection, Quantity):
-        pass
-    else:
-        raise ValueError(f'Cannot determine projection unit')
-
-    if extent.dimensionality != plonk_units('cm').dimensionality:
-        raise ValueError('extent has incorrect dimensions')
-    if projection.dimensionality != plonk_units('cm').dimensionality:
-        raise ValueError('projection has incorrect dimensions')
-
-    return {'quantity': quantity, 'extent': extent, 'projection': projection}
